import { APIEvent } from "@solidjs/start/server/types";
import { and, eq } from "drizzle-orm";
import db from "~/drizzle";
import { guilds, tpMessages } from "~/drizzle/schema";
import { BasicAuth } from "~/lib/auth";
import { ErrorResponse, Res } from "~/lib/responses";
import { zodId, zodTpMessages } from "~/lib/zod";
import { APIResponse, RequestBody } from "~/types/backend";

type Path = "/api/{guildId}/tp_messages";

const DayKeys = ["0", "1", "2", "3", "4", "5", "6"] as const;
type DayKeys = (typeof DayKeys)[number];
type Messages = Record<DayKeys, number | null>;

export const GET = async (
  event: APIEvent,
): Promise<APIResponse<Path, "get">> => {
  switch (event.request.headers.get("authorization")) {
    case BasicAuth.unencoded:
    case BasicAuth.encoded:
      break;

    default:
      return ErrorResponse("UNAUTHORIZED");
  }

  let guildId: number;
  try {
    guildId = zodId.parse(event.params.guildId);
  } catch (e) {
    return ErrorResponse("BAD_REQUEST", JSON.stringify(e));
  }

  const guild = await db.query.guilds.findFirst({
    where: eq(guilds.id, guildId),
    with: {
      tpMessages: true,
    },
  });

  if (!guild) return ErrorResponse("NOT_FOUND");

  if (!guild.tpEnabled || !guild.tpChannelId) return Res("NO_CONTENT", null);

  const tpMessages = guild.tpMessages.reduce(
    (acc, message) => {
      const day = message.day.toString() as DayKeys;
      if (!/^[0-6]$/.test(day)) return acc;
      acc[day] = message.messageId;
      return acc;
    },
    {
      "0": null,
      "1": null,
      "2": null,
      "3": null,
      "4": null,
      "5": null,
      "6": null,
    } as Messages,
  );

  return Res("OK", {
    channelId: guild.tpChannelId,
    messageIds: tpMessages,
  });
};

export const PUT = async (
  event: APIEvent,
): Promise<APIResponse<Path, "put">> => {
  switch (event.request.headers.get("authorization")) {
    case BasicAuth.unencoded:
    case BasicAuth.encoded:
      break;

    default:
      return ErrorResponse("UNAUTHORIZED");
  }

  let guildId: number;
  try {
    guildId = zodId.parse(event.params.guildId);
  } catch (e) {
    return ErrorResponse("BAD_REQUEST", JSON.stringify(e));
  }

  const guild = await db.query.guilds
    .findFirst({
      where: eq(guilds.id, guildId),
      with: { tpMessages: true },
    })
    .execute();

  if (!guild) return ErrorResponse("NOT_FOUND");

  if (!guild.tpEnabled) return ErrorResponse("FORBIDDEN");

  const unparsedBody = await new Response(event.request.body).json();

  let body: RequestBody<Path, "put">;
  try {
    body = zodTpMessages.parse(unparsedBody);
  } catch (e) {
    return ErrorResponse("BAD_REQUEST", JSON.stringify(e));
  }

  if (guild.tpChannelId !== body.channelId)
    await db
      .update(guilds)
      .set({ tpChannelId: body.channelId })
      .where(eq(guilds.id, guild.id))
      .execute();

  await Promise.all(
    DayKeys.map(async (dayStr) => {
      const day = parseInt(dayStr);
      await db
        .update(tpMessages)
        .set({ messageId: body.messageIds[dayStr] })
        .where(and(eq(tpMessages.guildId, guild.id), eq(tpMessages.day, day)))
        .execute();
    }),
  );

  return Res("NO_CONTENT", null);
};
